/*
 * Class:       Plocha.java
 *              Trida, kde se vytvari plocha pro hru a udrzuje se informace o vsech kartach.
 * Datum:       20. 5. 2011
 * Autor:       jandora1
 */

package jandora1;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;

/**
 *
 * @author jandora1
 */
public class Plocha {

    //vytvoreni prazdneho pole, vsechny prvky jsou null
    //pole neni dvojrozmerne, aby se nemusely pamatovat dve souradnice
    public Karta[] deska = new Karta[16];

    //uchovava prvni zapamatovani - pri prvnim otoceni se hodnota karty ulozi do
    // "klice" a do "hodnoty" se ulozi pozice
    public Map<Integer, Integer> map1 = new HashMap<Integer, Integer>();

    //uchovava pozici druhe otocene karty
    public Map<Integer, Integer> map2 = new HashMap<Integer, Integer>();
    public int body1 = 0;   //body hrace
    public int body2 = 0;   //body pocitace

    private boolean pocitac = false;

    //vytvori se plocha a naplni se kartama
    //mod8 protoze je 16 karet a potrebujeme je vytvorit v paru
    public Plocha() {
        for (int i = 0; i < deska.length; i++) {
            deska[i] = new Karta(i % 8, i);
        }
    }

    //vraci pole karet - jednorozmerny vektor
    public Karta[] getDeska() {
        return deska;
    }

    //vraci dvourozmerne pole
    public Karta[][] getCtverec() {        
        Karta[][] k = new Karta[4][4];
        for(int i = 0 ; i < deska.length ; i++) {
            k[i/4][i%4] = deska[i];
        }
        return k;
    }

    //nastaveni cele desky
    //public void setDeska(Karta[] deska) {
    //    this.deska = deska;
    //}

    //otoci kartu na zadane pozici
    //pokud je otoceno 2 a vice karet tak uz neotaci - vola se metoda getOctoceno
    public void otoc(int i) {
        if (getOtoceno() >= 2) {
            return;
        } else {
            deska[i].setOtoceno(true);
            if (getOtoceno() == 2) {
                napocitejSkore();
            }
        }
    }

    //otoci karty zpet obrazkem nahoru
    public void otocZpet() {
        for (Karta karta : deska) {
            karta.setOtoceno(false);
        }
        if (pocitac) {
            pocitac = false;
        } else {
            hrajPocitac();
        }

    }

    //vraci pocet otocenych karet
    public int getOtoceno() {
        int ret = 0;
        for (Karta karta : deska) {
            if (karta.isOtoceno()) {
                ret++;
            }
        }
        return ret;
    }

    //spocita skore a pripise body tomu hraje
    private void napocitejSkore() {
        Karta k1 = null;
        int pozice1 = -1;
        Karta k2 = null;
        int pozice2 = -1;
        for (int i = 0 ; i < deska.length ; i++) {
            if (deska[i].isOtoceno() && k1 == null) {
                k1 = deska[i];
                pozice1 = i;
            } else if (deska[i].isOtoceno() && k2 == null) {
                k2 = deska[i];
                pozice2 = i;
                break;
            }
        }
        if (k1.equals(k2)) {    //porovna, zda jsou dve otocene karty stejne
            if (pocitac) {      //testuje, zda hraje pocitac nebo hrac
                body2++;
            } else {
                body1++;
            }
            //vyradi karty, kdyz jsou stejne
            k1.setVyrazena(true);
            k2.setVyrazena(true);
            map1.remove(k1.getId());    //odstraneni karet
            map2.remove(k1.getId());    //z mapy znamejch karet

        //kdyz nejsou stejne, ulozi si je do map
        } else {
            if(map1.containsKey(k1.getId())) {                
                if(!map2.containsKey(k1.getId()) && map1.get(k1.getId()) != pozice1 ) {
                    map2.put(k1.getId(), pozice1);
                }
            }else {
                map1.put(k1.getId(), pozice1);
            }
            
            if(map1.containsKey(k2.getId()) && map1.get(k2.getId()) != pozice2) {
                if(!map2.containsKey(k2.getId())) {
                    map2.put(k2.getId(), pozice2);
                }
            }else {
                map1.put(k2.getId(), pozice2);
            }
        }
    }

    //vraci body pro hrace
    public int getBody1() {
        return body1;
    }

    //vraci body pro pocitac
    public int getBody2() {
        return body2;
    }

    //hraje pocitac = simulace tahu pocitace
    public void hrajPocitac() {
        pocitac = true;
        int pozice1 = -1;
        int pozice2 = -1;
        if(!map2.isEmpty()) {
            Iterator<Map.Entry<Integer, Integer>> it = map2.entrySet().iterator();
            it.hasNext();
            Map.Entry<Integer, Integer> item = it.next();
            pozice1 = item.getValue();  //umisteni prvni karty
            pozice2 = map1.get(item.getKey());  //v prvni mape karte vzdy existuje pokud je v 2.
            otoc(pozice1);
            otoc(pozice2);            
            return;         // konec 100% uspesneho tahu
        }

        pozice1 = getNahodnaPozice();
        otoc(pozice1);
        int hodnotaKarty = deska[pozice1].getId();
        if(map1.containsKey(hodnotaKarty) && map1.get(hodnotaKarty) != pozice1) {   //osetreni aby neotacel stejnou kartu
            pozice2 = map1.get(hodnotaKarty);
            otoc(pozice2);
            return;       // konec 100% uspesneho tahu s 1 nahodne zvolenou kartou
        }
        pozice2 = getNahodnaPozice();
        otoc(pozice2);
        return;
    }

    //vraci nahodnou pozici pocitaci, kdyz nevi, co hrat, tzn. nema v mapach ulozenou zadnou pozici karet
    public int getNahodnaPozice() {
        final int POCET_POKUSU = 100;
        Random r = new Random();

        int pozice = -1;
        int pokus = 0;
        do {
            pozice = r.nextInt(deska.length);
            pokus++;
        } while (
                (deska[pozice].isOtoceno() || deska[pozice].isVyrazena()) && pokus < POCET_POKUSU
                );
        
        if (!(pokus < POCET_POKUSU)) {
            for (pozice = 0; pozice < deska.length; pozice++) {
                if ( (!deska[pozice].isOtoceno()) && (!deska[pozice].isVyrazena())) {
                    return pozice;
                }
            }
            throw new IllegalArgumentException("vsechny karty otoceny nebo vyrazeny");            
        } else {
            return pozice;
        }
    }

    //vraci priznak, ze hral pocitac
    public boolean isPocitac() {
        return pocitac;
    }

    //ulozeni pozic do map1
    public Map<Integer, Integer> getMap1() {
        return map1;
    }

    //ulozeni pozic do map2
    public Map<Integer, Integer> getMap2() {
        return map2;
    }
}
